@@ -16,15 +16,21 @@ from re import compile as re_compile, escape as re_escape, Pattern
 from subprocess import run
 from threading import Thread
 from typing import cast, Any, AnyStr, Final, Optional, TypedDict
-from urllib.parse import parse_qsl, urlsplit, unquote
+from urllib.parse import parse_qsl, urlencode, urlsplit, unquote
 from uuid import uuid4
 
 from ddddocr import DdddOcr # type: ignore
+from lxml.html import fromstring, tostring, HtmlElement
 from qrcode import QRCode # type: ignore
 from requests import Session, get
 from requests.cookies import create_cookie
 
 
+# 百度网盘 openapi 的应用，直接使用 AList 的
+# https://alist.nn.ci/guide/drivers/baidu.html
+CLIENT_ID = "iYCeC9g08h5vuP9UqvPHKKSVrKFXGa1v"
+CLIENT_SECRET = "jXiFMOPVPCWlO2M5CwWQzffpNPaGTRBG"
+# 百度网盘 errno 对应的信息
 ERRNO_TO_MESSAGE: Final[dict[int, str]] = {
     0: "成功", 
     -1: "由于您分享了违反相关法律法规的文件，分享功能已被禁用，之前分享出去的文件不受影响。", 
@@ -72,9 +78,9 @@ class VCodeResult(TypedDict, total=True):
     vcode_str: str
 
 
-def recognize_vcode(_ocr=DdddOcr(beta=True), /) -> VCodeResult:
+def recognize_vcode(_ocr=DdddOcr(beta=True, show_ad=False), /) -> VCodeResult:
     "识别百度网盘的验证码"
-    url = "https://pan.baidu.com/api/getcaptcha?prod=shareverify&web=1&DuPanClienttype=0&bdstoken="
+    url = "https://pan.baidu.com/api/getcaptcha?prod=shareverify&web=1&clienttype=0&bdstoken="
     while True:
         resp = get(url).json()
         for i in range(10):
@@ -196,7 +202,7 @@ class DuPanClient:
         resp = self.get_qrcode(gid)
         sign = resp["sign"]
         if scan_in_console:
-            url = url = f"https://wappass.baidu.com/wp/?qrlogin&error=0&sign={sign}&cmd=login&lp=pc&tpl=netdisk&adapter=3&qrloginfrom=pc"
+            url = f"https://wappass.baidu.com/wp/?qrlogin&error=0&sign={sign}&cmd=login&lp=pc&tpl=netdisk&adapter=3&qrloginfrom=pc"
             console_qrcode(url)
         else:
             imgurl = "https://" + resp["imgurl"]
@@ -275,7 +281,7 @@ class DuPanClient:
 
     @staticmethod
     def list_app_version():
-        url = "https://pan.baidu.com/disk/cmsdata?DuPanClienttype=0&web=1&do=DuPanClient"
+        url = "https://pan.baidu.com/disk/cmsdata?clienttype=0&web=1&do=client"
         return get(url).json()
 
     def get_qrcode(self, /, gid):
@@ -326,14 +332,14 @@ class DuPanClient:
             page: int = 1
             order: str = "time"
             desc: 0 | 1 = 1
-            DuPanClienttype: int = 0
+            clienttype: int = 0
             web: int = 1
         """
         api = "https://pan.baidu.com/api/list"
         if isinstance(payload, str):
-            payload = {"num": 100, "page": 1, "order": "time", "desc": 1, "DuPanClienttype": 0, "web": 1, "dir": payload}
+            payload = {"num": 100, "page": 1, "order": "time", "desc": 1, "clienttype": 0, "web": 1, "dir": payload}
         else:
-            payload = {"num": 100, "page": 1, "order": "time", "desc": 1, "DuPanClienttype": 0, "web": 1, **payload}
+            payload = {"num": 100, "page": 1, "order": "time", "desc": 1, "clienttype": 0, "web": 1, **payload}
         return self.request(api, params=payload)
 
     def makedir(
@@ -345,7 +351,7 @@ class DuPanClient:
         params = {
             "a": "commit", 
             "bdstoken": self.bdstoken, 
-            "DuPanClienttype": 0, 
+            "clienttype": 0, 
             "web": 1, 
         }
         if isinstance(payload, str):
@@ -368,7 +374,7 @@ class DuPanClient:
             "onnest": "fail", 
             "bdstoken": self.bdstoken, 
             "newVerify": 1, 
-            "DuPanClienttype": 0, 
+            "clienttype": 0, 
             "web": 1, 
             **params, 
         }
@@ -416,7 +422,7 @@ class DuPanClient:
     ) -> dict:
         api = "https://pan.baidu.com/share/taskquery"
         if isinstance(payload, (int, str)):
-            payload = {"DuPanClienttype": 0, "web": 1, "taskid": payload}
+            payload = {"clienttype": 0, "web": 1, "taskid": payload}
         return self.request(api, params=payload)
 
     def transfer(
@@ -432,7 +438,7 @@ class DuPanClient:
             "async": 1, 
             "bdstoken": self.bdstoken, 
             "web": 1, 
-            "DuPanClienttype": 0, 
+            "clienttype": 0, 
             **params, 
         }
         if isinstance(data, (int, str)):
@@ -443,16 +449,85 @@ class DuPanClient:
             data["fsidlist"] = "[%s]" % ",".join(map(str, data["fsidlist"]))
         return self.request(api, "POST", params=params, data=data, headers={"Referer": url})
 
+    def oauth_authorize(
+        self, 
+        /, 
+        client_id: str = CLIENT_ID, 
+        scope: str = "basic,netdisk", 
+    ) -> str:
+        def check(etree):
+            error_msg = etree.find_class("error-msg-list")
+            if error_msg:
+                raise OSError(tostring(error_msg[0], encoding="utf-8").decode("utf-8").strip())
+        api = "https://openapi.baidu.com/oauth/2.0/authorize"
+        params = {
+            "response_type": "code", 
+            "client_id": client_id, 
+            "redirect_uri": "oob", 
+            "scope": scope, 
+            "display": "popup", 
+        }
+        resp = self.request(api, params=params)
+        etree: HtmlElement = fromstring(resp)
+        check(etree)
+        try:
+            return etree.get_element_by_id("Verifier").value
+        except KeyError:
+            pass
+        payload = []
+        grant_permissions = []
+        el: HtmlElement
+        for el in fromstring(resp).cssselect('form[name="scopes"] input'):
+            name, value = el.name, el.value
+            if name == "grant_permissions_arr":
+                grant_permissions.append(value)
+                payload.append(("grant_permissions_arr[]", value))
+            elif name == "grant_permissions":
+                payload.append(("grant_permissions", ",".join(grant_permissions)))
+            else:
+                payload.append((name, value))
+        resp = self.request(
+            api, 
+            "POST", 
+            params=params, 
+            data=urlencode(payload), 
+            headers={"Content-Type": "application/x-www-form-urlencoded"}
+        )
+        etree = fromstring(resp)
+        return etree.get_element_by_id("Verifier").value
+
+    def oauth_token(
+        self, 
+        /, 
+        client_id: str = CLIENT_ID, 
+        client_secret: str = CLIENT_SECRET, 
+        scope: str = "basic,netdisk", 
+    ) -> dict:
+        api = "https://openapi.baidu.com/oauth/2.0/token"
+        params = {
+            "grant_type": "authorization_code", 
+            "code": self.oauth_authorize(client_id, scope), 
+            "client_id": client_id, 
+            "client_secret": client_secret, 
+            "redirect_uri": "oob", 
+        }
+        return self.request(api, params=params)
+
 
 class DuPanShareList:
 
-    def __init__(self, url):
+    def __init__(self, url: str, password: str = ""):
         if url.startswith(("http://", "https://")):
-            shorturl, password = self._extract_from_url(url)
+            shorturl, _password = self._extract_from_url(url)
+            if not password:
+                password = _password
+            # NOTE: OR url = f"https://pan.baidu.com/s/1{shorturl}"
+            if shorturl:
+                url = f"https://pan.baidu.com/share/init?surl={shorturl}"
         else:
             shorturl = url
-            password = ""
-        url = self.url = f"https://pan.baidu.com/share/init?surl={shorturl}"
+            url = f"https://pan.baidu.com/share/init?surl={shorturl}"
+        self.url = url
         self.shorturl = shorturl
         self.password = password
         session = self.session = Session()
@@ -463,7 +538,9 @@ class DuPanShareList:
         urlp = urlsplit(url)
         path = urlp.path
         query = dict(parse_qsl(urlp.query))
-        if path == "/share/init":
+        if path == "/share/link":
+            shorturl = ""
+        elif path == "/share/init":
             shorturl = query["surl"]
         elif path.startswith("/s/1"):
             shorturl = path.removeprefix("/s/1")
@@ -488,6 +565,11 @@ class DuPanShareList:
         self.listdir_index()
         return self.__dict__["root"]
 
+    @cached_property
+    def randsk(self, /) -> str:
+        self.listdir_index()
+        return unquote(self.session.cookies.get("BDCLND", ""))
+
     @cached_property
     def share_id(self, /):
         self.listdir_index()
@@ -498,9 +580,9 @@ class DuPanShareList:
         self.listdir_index()
         return self.__dict__["share_uk"]
 
-    def verify(self, /) -> str:
+    def verify(self, /):
         api = "https://pan.baidu.com/share/verify"
-        params: dict[str, int | str] = {"surl": self.shorturl, "web": 1, "DuPanClienttype": 0}
+        params: dict[str, int | str] = {"surl": self.shorturl, "web": 1, "clienttype": 0}
         data = {"pwd": self.password}
         session = self.session
         while True:
@@ -514,10 +596,6 @@ class DuPanShareList:
                     data.update(cast(dict[str, str], recognize_vcode()))
                 else:
                     raise OSError(json)
-        randsk = json["randsk"]
-        session.cookies.set("BDCLND", randsk, domain=".baidu.com")
-        self.__dict__["randsk"] = unquote(randsk)
-        return randsk
 
     def listdir_index(self, /) -> dict:
         url = self.url
@@ -561,7 +639,7 @@ class DuPanShareList:
             "order": "other", 
             "desc": 1, 
             "showempty": 0, 
-            "DuPanClienttype": 0, 
+            "clienttype": 0, 
             "web": 1, 
             "page": 1, 
             "num": 100, 
